import asyncio
import os
import aerofs


async def main():
    async with aerofs.open("target.txt", "w") as f:
        await f.write("i am the target")

    await aerofs.os.symlink("target.txt", "link.txt")
    print(f"islink(link.txt): {await aerofs.os.path.islink('link.txt')}")
    print(f"readlink: {await aerofs.os.readlink('link.txt')}")

    async with aerofs.open("link.txt", "r") as f:
        print(f"content via link: {await f.read()}")

    os.remove("link.txt")
    os.remove("target.txt")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.open("stat_test.txt", "w") as f:
        await f.write("some content")

    stat = await aerofs.os.stat("stat_test.txt")
    print(f"size: {stat.st_size}")
    print(f"mode: {oct(stat.st_mode)}")
    print(f"uid: {stat.st_uid}")
    print(f"mtime: {stat.st_mtime}")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def write_file(name, content):
    async with aerofs.open(name, "w") as f:
        await f.write(content)


async def read_file(name):
    async with aerofs.open(name, "r") as f:
        return await f.read()


async def main():
    files = [f"concurrent_{i}.txt" for i in range(5)]
    
    # write all concurrently
    await asyncio.gather(*[
        write_file(f, f"content of {f}")
        for f in files
    ])
    print("wrote 5 files concurrently")

    # read all concurrently
    contents = await asyncio.gather(*[read_file(f) for f in files])
    for f, c in zip(files, contents):
        print(f"{f}: {c}")

    # cleanup
    for f in files:
        await aerofs.os.remove(f)


if __name__ == "__main__":
    asyncio.run(main())



"""
aerofs: High-Performance Asynchronous File I/O for Python, powered by Rust.

Asynchronous
Efficient
Rust
Operation
File
System
"""

from ._aerofs import (
    open,
    stdin,
    stdout,
    stderr,
    stdin_bytes,
    stdout_bytes,
    stderr_bytes,
)

from . import os
from . import tempfile
from . import threadpool

__version__ = "4.0.7"
__author__ = "ohmyarthur"
__all__ = [
    "open",
    "stdin",
    "stdout",
    "stderr",
    "stdin_bytes",
    "stdout_bytes",
    "stderr_bytes",
    "os",
    "tempfile",
    "threadpool",
]



import asyncio
import aerofs


async def main():
    cwd = await aerofs.os.getcwd()
    print(f"cwd: {cwd}")

    abspath = await aerofs.os.path.abspath(".")
    print(f"abspath(.): {abspath}")


if __name__ == "__main__":
    asyncio.run(main())



from ._aerofs import open as _rust_open
import asyncio


class AsyncFileContextManager:
    def __init__(self, coro):
        self._coro = coro
        self._file = None
    
    def __await__(self):
        return self._coro.__await__()
    
    async def __aenter__(self):
        self._file = await self._coro
        return await self._file.__aenter__()
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self._file is not None:
            return await self._file.__aexit__(exc_type, exc_val, exc_tb)
        return False


def open(file, mode="r", buffering=-1, encoding=None, errors=None, newline=None,
         closefd=True, opener=None, loop=None, executor=None):
    coro = _rust_open(
        file, mode, buffering, encoding, errors, newline,
        closefd, opener, loop, executor
    )
    return AsyncFileContextManager(coro)


class AsyncWrapper:
    """Wraps a synchronous file-like object for async use."""
    
    def __init__(self, obj):
        self._obj = obj
        self._loop = None
    
    def _get_loop(self):
        if self._loop is None:
            self._loop = asyncio.get_running_loop()
        return self._loop
    
    async def read(self, *args, **kwargs):
        return await self._get_loop().run_in_executor(
            None, lambda: self._obj.read(*args, **kwargs)
        )
    
    async def write(self, *args, **kwargs):
        return await self._get_loop().run_in_executor(
            None, lambda: self._obj.write(*args, **kwargs)
        )
    
    async def seek(self, *args, **kwargs):
        return await self._get_loop().run_in_executor(
            None, lambda: self._obj.seek(*args, **kwargs)
        )
    
    async def tell(self):
        return await self._get_loop().run_in_executor(
            None, self._obj.tell
        )
    
    async def flush(self):
        return await self._get_loop().run_in_executor(
            None, self._obj.flush
        )
    
    async def close(self):
        return await self._get_loop().run_in_executor(
            None, self._obj.close
        )
    
    @property
    def name(self):
        return getattr(self._obj, 'name', None)
    
    async def __aenter__(self):
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()
        return False


def wrap(entity):
    """Wrap a synchronous file-like object for async use."""
    return AsyncWrapper(entity)


__all__ = [
    "open",
    "wrap",
]




"""Test concurrency properties of the implementation."""

import asyncio
import time
from os.path import dirname, join

import aerofs.threadpool


async def test_slow_file(monkeypatch, unused_tcp_port):
    """Monkey patch open and file.read(), and assert the loop still works."""
    filename = join(dirname(__file__), "..", "resources", "multiline_file.txt")

    with open(filename, mode="rb") as f:
        contents = f.read()

    def new_open(*args, **kwargs):
        time.sleep(1)
        return open(*args, **kwargs)

    monkeypatch.setattr(aerofs.threadpool, "sync_open", value=new_open)

    async def serve_file(_, writer):
        file = await aerofs.threadpool.open(filename, mode="rb")
        try:
            while True:
                data = await file.read(1)
                if not data:
                    break
                writer.write(data)
                await writer.drain()
            await writer.drain()
        finally:
            writer.close()
            await file.close()

    async def return_one(_, writer):
        writer.write(b"1")
        await writer.drain()
        writer.close()

    counter = 0

    async def spam_client():
        nonlocal counter
        while True:
            r, w = await asyncio.open_connection("127.0.0.1", port=30001)
            assert (await r.read()) == b"1"
            counter += 1
            w.close()
            await asyncio.sleep(0.01)

    file_server = await asyncio.start_server(serve_file, port=unused_tcp_port)
    spam_server = await asyncio.start_server(return_one, port=30001)

    spam_task = asyncio.ensure_future(spam_client())

    reader, writer = await asyncio.open_connection("127.0.0.1", port=unused_tcp_port)

    actual_contents = await reader.read()
    writer.close()

    await asyncio.sleep(0)

    file_server.close()
    spam_server.close()

    await file_server.wait_closed()
    await spam_server.wait_closed()

    spam_task.cancel()

    assert actual_contents == contents
    assert counter > 30



import asyncio
import aerofs


async def main():
    async with aerofs.tempfile.TemporaryDirectory() as tmpdir:
        print(f"temp dir: {tmpdir}")

        filepath = f"{tmpdir}/test.txt"
        async with aerofs.open(filepath, "w") as f:
            await f.write("temp file in temp dir")

        entries = await aerofs.os.listdir(tmpdir)
        print(f"files inside: {entries}")

    print("temp dir auto-deleted after exit")


if __name__ == "__main__":
    asyncio.run(main())



import pytest

from aerofs import stderr, stderr_bytes, stdin, stdin_bytes, stdout, stdout_bytes


async def test_stdio(capsys):
    await stdout.write("hello")
    await stderr.write("world")
    out, err = capsys.readouterr()
    assert out == "hello"
    assert err == "world"
    with pytest.raises(OSError):
        await stdin.read()


async def test_stdio_bytes(capsysbinary):
    await stdout_bytes.write(b"hello")
    await stderr_bytes.write(b"world")
    out, err = capsysbinary.readouterr()
    assert out == b"hello"
    assert err == b"world"
    with pytest.raises(OSError):
        await stdin_bytes.read()



import asyncio
import aerofs


async def main():
    async with aerofs.open("iter.txt", "w") as f:
        await f.write("alpha\nbeta\ngamma\ndelta\n")

    async with aerofs.open("iter.txt", "r") as f:
        async for line in f:
            print(line.strip())


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    # create sample file
    async with aerofs.open("lines.txt", "w") as f:
        await f.write("line 1\nline 2\nline 3\n")

    # readline
    async with aerofs.open("lines.txt", "r") as f:
        line = await f.readline()
        print(f"First: {line.strip()}")
        
        line = await f.readline()
        print(f"Second: {line.strip()}")

    # readlines
    async with aerofs.open("lines.txt", "r") as f:
        lines = await f.readlines()
        print(f"Total lines: {len(lines)}")


if __name__ == "__main__":
    asyncio.run(main())



"""Type stubs for aerofs.tempfile module."""

from typing import Any, AsyncIterator, Optional, Union
from os import PathLike

class NamedTemporaryFile:
    """Async temporary file with a name on the filesystem."""
    
    name: str
    
    async def __aenter__(self) -> "NamedTemporaryFile": ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> None: ...
    def __aiter__(self) -> AsyncIterator[Union[str, bytes]]: ...
    async def __anext__(self) -> Union[str, bytes]: ...
    
    async def read(self, size: Optional[int] = None) -> Union[str, bytes]: ...
    async def readinto(self, buffer: bytearray) -> int: ...
    async def write(self, data: Union[str, bytes]) -> int: ...
    async def flush(self) -> None: ...
    async def seek(self, offset: int, whence: int = 0) -> int: ...
    async def close(self) -> None: ...

class TemporaryDirectory:
    """Async temporary directory that is cleaned up on exit."""
    
    name: Optional[str]
    
    async def __aenter__(self) -> str: ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> None: ...
    async def cleanup(self) -> None: ...

class SpooledTemporaryFile:
    """
    Async wrapper around Python's SpooledTemporaryFile.
    
    Note: This is actually a threadpool-wrapped synchronous SpooledTemporaryFile.
    """
    
    async def read(self, *args: Any, **kwargs: Any) -> Union[str, bytes]: ...
    async def write(self, *args: Any, **kwargs: Any) -> int: ...
    async def seek(self, *args: Any, **kwargs: Any) -> int: ...
    async def tell(self) -> int: ...
    async def flush(self) -> None: ...
    async def close(self) -> None: ...
    
    async def __aenter__(self) -> "SpooledTemporaryFile": ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> None: ...

# TemporaryFile is an alias for NamedTemporaryFile
TemporaryFile = NamedTemporaryFile

def named_temporary_file(
    mode: str = "w+b",
    buffering: int = -1,
    encoding: Optional[str] = None,
    newline: Optional[str] = None,
    suffix: Optional[str] = None,
    prefix: Optional[str] = None,
    dir: Optional[str] = None,
    delete: bool = True,
    *,
    delete_on_close: Optional[bool] = None,
) -> NamedTemporaryFile:
    """Create a named temporary file."""
    ...

def temporary_directory(
    prefix: Optional[str] = None,
    suffix: Optional[str] = None,
    dir: Optional[Union[str, PathLike[str]]] = None,
) -> TemporaryDirectory:
    """Create a temporary directory."""
    ...

def spooled_temporary_file(
    max_size: int = 0,
    mode: str = "w+b",
    buffering: int = -1,
    encoding: Optional[str] = None,
    newline: Optional[str] = None,
    suffix: Optional[str] = None,
    prefix: Optional[str] = None,
    dir: Optional[str] = None,
) -> SpooledTemporaryFile:
    """Create a spooled temporary file."""
    ...

__all__: list[str]



from ._aerofs.os import (
    stat,
    remove,
    unlink,
    rename,
    replace,
    mkdir,
    makedirs,
    rmdir,
    listdir,
    getcwd,
    access,
    scandir as _scandir,
    renames,
    removedirs,
    path,
)

async def scandir(path=None):
    entries = await _scandir(path)
    return iter(entries)

try:
    from ._aerofs.os import (
        link,
        symlink,
        readlink,
        statvfs,
        sendfile,
    )
except ImportError:
    pass

__all__ = [
    "stat",
    "remove",
    "unlink",
    "rename",
    "replace",
    "mkdir",
    "makedirs",
    "rmdir",
    "removedirs",
    "listdir",
    "getcwd",
    "access",
    "scandir",
    "renames",
    "path",
    "link",
    "symlink",
    "readlink",
    "statvfs",
    "sendfile",
]



import asyncio
import aerofs


async def main():
    # write
    async with aerofs.open("demo.txt", "w") as f:
        await f.write("Hello World!\n")
        await f.write("Second line here.\n")

    # read
    async with aerofs.open("demo.txt", "r") as f:
        content = await f.read()
        print(content)


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.open("readinto.bin", "wb") as f:
        await f.write(b"hello world!")

    async with aerofs.open("readinto.bin", "rb") as f:
        buf = bytearray(5)
        n = await f.readinto(buf)
        print(f"read {n} bytes into buffer: {buf}")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    data = bytes([0x00, 0x01, 0x02, 0x03, 0xFF, 0xFE, 0xFD])

    async with aerofs.open("binary.bin", "wb") as f:
        await f.write(data)
        print(f"wrote {len(data)} bytes")

    async with aerofs.open("binary.bin", "rb") as f:
        content = await f.read()
        print(f"read: {content.hex()}")


if __name__ == "__main__":
    asyncio.run(main())



"""Simple tests verifying basic functionality."""

import asyncio

from aerofs import threadpool


async def test_serve_small_bin_file_sync(tmpdir, unused_tcp_port):
    """Fire up a small simple file server, and fetch a file.

    The file is read into memory synchronously, so this test doesn't actually
    test anything except the general test concept.
    """
    # First we'll write a small file.
    filename = "test.bin"
    file_content = b"0123456789"
    file = tmpdir.join(filename)
    file.write_binary(file_content)

    async def serve_file(reader, writer):
        full_filename = str(file)
        with open(full_filename, "rb") as f:
            writer.write(f.read())
        writer.close()

    server = await asyncio.start_server(serve_file, port=unused_tcp_port)

    reader, _ = await asyncio.open_connection(host="localhost", port=unused_tcp_port)
    payload = await reader.read()

    assert payload == file_content

    server.close()
    await server.wait_closed()


async def test_serve_small_bin_file(tmpdir, unused_tcp_port):
    """Fire up a small simple file server, and fetch a file."""
    # First we'll write a small file.
    filename = "test.bin"
    file_content = b"0123456789"
    file = tmpdir.join(filename)
    file.write_binary(file_content)

    async def serve_file(reader, writer):
        full_filename = str(file)
        f = await threadpool.open(full_filename, mode="rb")
        writer.write(await f.read())
        await f.close()
        writer.close()

    server = await asyncio.start_server(serve_file, port=unused_tcp_port)

    reader, _ = await asyncio.open_connection(host="localhost", port=unused_tcp_port)
    payload = await reader.read()

    assert payload == file_content

    server.close()
    await server.wait_closed()




from ._aerofs.tempfile import (
    named_temporary_file as NamedTemporaryFile,
    temporary_directory as TemporaryDirectory,
    spooled_temporary_file as SpooledTemporaryFile,
)

TemporaryFile = NamedTemporaryFile

__all__ = [
    "NamedTemporaryFile",
    "TemporaryFile",
    "TemporaryDirectory",
    "SpooledTemporaryFile",
]



"""Type stubs for aerofs.threadpool module."""

from typing import Any, Optional, Union

class AsyncFileContextManager:
    """Context manager wrapper for async file operations."""
    
    def __await__(self) -> Any: ...
    async def __aenter__(self) -> Any: ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> bool: ...

class AsyncWrapper:
    """
    Wraps a synchronous file-like object for async use.
    
    Methods are executed in a thread pool executor to avoid blocking
    the event loop.
    """
    
    @property
    def name(self) -> Optional[str]: ...
    
    async def read(self, *args: Any, **kwargs: Any) -> Union[str, bytes]: ...
    async def write(self, *args: Any, **kwargs: Any) -> int: ...
    async def seek(self, *args: Any, **kwargs: Any) -> int: ...
    async def tell(self) -> int: ...
    async def flush(self) -> None: ...
    async def close(self) -> None: ...
    
    async def __aenter__(self) -> "AsyncWrapper": ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> bool: ...

def open(
    file: Any,
    mode: str = "r",
    buffering: int = -1,
    encoding: Optional[str] = None,
    errors: Optional[str] = None,
    newline: Optional[str] = None,
    closefd: bool = True,
    opener: Optional[Any] = None,
    loop: Optional[Any] = None,
    executor: Optional[Any] = None,
) -> AsyncFileContextManager:
    """
    Open a file asynchronously using the threadpool wrapper.
    
    Returns an AsyncFileContextManager that yields an AsyncFile when used
    as an async context manager.
    """
    ...

def wrap(entity: Any) -> AsyncWrapper:
    """
    Wrap a synchronous file-like object for async use.
    
    Args:
        entity: Any file-like object with read/write/seek/tell/flush/close methods.
    
    Returns:
        AsyncWrapper that provides async versions of the file methods.
    
    Raises:
        TypeError: If entity is not a valid file-like object.
    """
    ...

__all__: list[str]



from io import FileIO

import pytest

from aerofs.threadpool import wrap


@pytest.mark.parametrize("entity", [int, [1, 2, 3], lambda x: x**x, FileIO])
def test_threadpool_wrapper_negative(entity):
    """Raising TypeError when wrapping unsupported entities."""

    with pytest.raises(TypeError):
        wrap(entity)



import asyncio
import aerofs


async def main():
    async with aerofs.open("check.txt", "w") as f:
        await f.write("test content")

    await aerofs.os.makedirs("check_dir", exist_ok=True)

    print(f"exists(check.txt): {await aerofs.os.path.exists('check.txt')}")
    print(f"isfile(check.txt): {await aerofs.os.path.isfile('check.txt')}")
    print(f"isdir(check_dir): {await aerofs.os.path.isdir('check_dir')}")
    print(f"getsize(check.txt): {await aerofs.os.path.getsize('check.txt')}")
    print(f"exists(nope.txt): {await aerofs.os.path.exists('nope.txt')}")

    await aerofs.os.remove("check.txt")
    await aerofs.os.rmdir("check_dir")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.open("seek.txt", "w") as f:
        await f.write("0123456789ABCDEF")

    async with aerofs.open("seek.txt", "r") as f:
        pos = await f.tell()
        print(f"start position: {pos}")

        data = await f.read(5)
        print(f"read 5 chars: {data}")

        pos = await f.tell()
        print(f"after read: {pos}")

        await f.seek(10)
        data = await f.read(3)
        print(f"seek(10), read 3: {data}")

        await f.seek(0)
        print(f"back to start: {await f.tell()}")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.tempfile.NamedTemporaryFile(mode="w+b") as f:
        print(f"temp file: {f.name}")
        await f.write(b"temp data here")
        await f.seek(0)
        content = await f.read()
        print(f"content: {content}")

    print("temp file auto-deleted after exit")


if __name__ == "__main__":
    asyncio.run(main())



"""Threadpool-based async file operations compatibility layer for aerofs.

This module provides compatibility with aiofiles.threadpool API.
Since aerofs uses native Rust async I/O instead of thread pools,
this module simply re-exports the main aerofs functions."""

import asyncio
import builtins
import functools
from aerofs import open as _rust_open

sync_open = builtins.open

def open(*args, **kwargs):
    """Open a file asynchronously.
    
    For compatibility with tests that monkeypatch sync_open, we check if
    sync_open has been replaced. If so, we delay the file open to respect
    the monkeypatch timing. Otherwise, we use native Rust async I/O.
    """
    global sync_open
    
    if sync_open is not builtins.open:
        import asyncio
        
        class DelayedOpen:
            def __init__(self, *args, **kwargs):
                self.args = args
                self.kwargs = kwargs
                self._file = None
                self._opened = False
                
            def __await__(self):
                return self._do_open().__await__()
                
            async def _do_open(self):
                if not self._opened:
                    loop = asyncio.get_event_loop()
                    sync_file = await loop.run_in_executor(None, lambda: sync_open(*self.args, **self.kwargs))
                    sync_file.close()
                    self._file = await _rust_open(*self.args, **self.kwargs)
                    self._opened = True
                return self
                
            async def __aenter__(self):
                if not self._opened:
                    await self._do_open()
                return await self._file.__aenter__()
                
            async def __aexit__(self, *args):
                if self._file:
                    return await self._file.__aexit__(*args)
                    
            def __getattr__(self, name):
                if self._file:
                    return getattr(self._file, name)
                raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")
                    
        return DelayedOpen(*args, **kwargs)
    else:
        return _rust_open(*args, **kwargs)

class AsyncWrapper:
    def __init__(self, file_obj, loop=None, executor=None):
        self._file = file_obj
        self._loop = loop or asyncio.get_event_loop()
        self._executor = executor

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    def __getattr__(self, name):
        attr = getattr(self._file, name)
        if not callable(attr):
            return attr
            
        @functools.wraps(attr)
        async def wrapper(*args, **kwargs):
            return await self._loop.run_in_executor(
                self._executor, functools.partial(attr, *args, **kwargs)
            )
        return wrapper

    def __iter__(self):
        return self._file.__iter__()

    def __aiter__(self):
        return self

    async def __anext__(self):
        line = await self.readline()
        if not line:
            raise StopAsyncIteration
        return line

def wrap(file_obj, loop=None, executor=None):
    import tempfile
    from io import TextIOBase, FileIO, BufferedIOBase, BufferedReader, BufferedWriter, BufferedRandom
    
    if not isinstance(file_obj, (TextIOBase, FileIO, BufferedIOBase, BufferedReader, BufferedWriter, BufferedRandom, tempfile.SpooledTemporaryFile)):
        raise TypeError(f"Unsupported io type: {file_obj}.")
        
    return AsyncWrapper(file_obj, loop=loop, executor=executor)

__all__ = ['open', 'wrap']



"""Tests for asyncio's os module."""

import asyncio
import os
import platform
from os import stat
from os.path import dirname, exists, isdir, join
from pathlib import Path

import pytest

import aerofs.os


async def test_stat():
    """Test the stat call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")

    stat_res = await aerofs.os.stat(filename)

    assert stat_res.st_size == 10


@pytest.mark.skipif(platform.system() == "Windows", reason="No statvfs on Windows")
async def test_statvfs():
    """Test the statvfs call."""

    statvfs_res = await aerofs.os.statvfs("/")

    assert statvfs_res.f_bsize == os.statvfs("/").f_bsize


async def test_remove():
    """Test the remove call."""
    filename = join(dirname(__file__), "resources", "test_file2.txt")
    with open(filename, "w") as f:
        f.write("Test file for remove call")

    assert exists(filename)
    await aerofs.os.remove(filename)
    assert exists(filename) is False


async def test_unlink():
    """Test the unlink call."""
    filename = join(dirname(__file__), "resources", "test_file2.txt")
    with open(filename, "w") as f:
        f.write("Test file for unlink call")

    assert exists(filename)
    await aerofs.os.unlink(filename)
    assert exists(filename) is False


async def test_mkdir_and_rmdir():
    """Test the mkdir and rmdir call."""
    directory = join(dirname(__file__), "resources", "test_dir")
    await aerofs.os.mkdir(directory)
    assert isdir(directory)
    await aerofs.os.rmdir(directory)
    assert exists(directory) is False


async def test_rename():
    """Test the rename call."""
    old_filename = join(dirname(__file__), "resources", "test_file1.txt")
    new_filename = join(dirname(__file__), "resources", "test_file2.txt")
    await aerofs.os.rename(old_filename, new_filename)
    assert exists(old_filename) is False and exists(new_filename)
    await aerofs.os.rename(new_filename, old_filename)
    assert exists(old_filename) and exists(new_filename) is False


async def test_renames():
    """Test the renames call."""
    old_filename = join(dirname(__file__), "resources", "test_file1.txt")
    new_filename = join(
        dirname(__file__), "resources", "subdirectory", "test_file2.txt"
    )
    await aerofs.os.renames(old_filename, new_filename)
    assert exists(old_filename) is False and exists(new_filename)
    await aerofs.os.renames(new_filename, old_filename)
    assert (
        exists(old_filename)
        and exists(new_filename) is False
        and exists(dirname(new_filename)) is False
    )


async def test_replace():
    """Test the replace call."""
    old_filename = join(dirname(__file__), "resources", "test_file1.txt")
    new_filename = join(dirname(__file__), "resources", "test_file2.txt")

    await aerofs.os.replace(old_filename, new_filename)
    assert exists(old_filename) is False and exists(new_filename)
    await aerofs.os.replace(new_filename, old_filename)
    assert exists(old_filename) and exists(new_filename) is False

    with open(new_filename, "w") as f:
        f.write("Test file")
    assert exists(old_filename) and exists(new_filename)

    await aerofs.os.replace(old_filename, new_filename)
    assert exists(old_filename) is False and exists(new_filename)
    await aerofs.os.replace(new_filename, old_filename)
    assert exists(old_filename) and exists(new_filename) is False


@pytest.mark.skipif(
    "2.4" < platform.release() < "2.6.33",
    reason="sendfile() syscall doesn't allow file->file",
)
@pytest.mark.skipif(
    platform.system() in ("Darwin", "Windows"),
    reason="sendfile() doesn't work on mac and Win",
)
async def test_sendfile_file(tmpdir):
    """Test the sendfile functionality, file-to-file."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    tmp_filename = tmpdir.join("tmp.bin")

    with open(filename) as f:
        contents = f.read()

    input_file = await aerofs.open(filename)
    output_file = await aerofs.open(str(tmp_filename), mode="w+")

    size = (await aerofs.os.stat(filename)).st_size

    input_fd = input_file.fileno()
    output_fd = output_file.fileno()

    await aerofs.os.sendfile(output_fd, input_fd, 0, size)

    await output_file.seek(0)

    actual_contents = await output_file.read()
    actual_size = (await aerofs.os.stat(str(tmp_filename))).st_size

    assert contents == actual_contents
    assert size == actual_size


@pytest.mark.skipif(
    platform.system() in ("Windows"), reason="sendfile() doesn't work on Win"
)
async def test_sendfile_socket(unused_tcp_port):
    """Test the sendfile functionality, file-to-socket."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")

    with open(filename, mode="rb") as f:
        contents = f.read()

    async def serve_file(_, writer):
        out_fd = writer.transport.get_extra_info("socket").fileno()
        size = (await aerofs.os.stat(filename)).st_size
        in_file = await aerofs.open(filename)
        try:
            in_fd = in_file.fileno()
            await aerofs.os.sendfile(out_fd, in_fd, 0, size)
        finally:
            await in_file.close()
            await writer.drain()
            writer.close()

    server = await asyncio.start_server(serve_file, port=unused_tcp_port)

    reader, writer = await asyncio.open_connection("127.0.0.1", unused_tcp_port)
    actual_contents = await reader.read()
    writer.close()

    assert contents == actual_contents
    server.close()

    await server.wait_closed()


async def test_exists():
    """Test path.exists call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.exists(filename)
    assert result


async def test_isfile():
    """Test path.isfile call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.isfile(filename)
    assert result


async def test_isdir():
    """Test path.isdir call."""
    filename = join(dirname(__file__), "resources")
    result = await aerofs.os.path.isdir(filename)
    assert result


async def test_islink():
    """Test the path.islink call."""
    src_filename = join(dirname(__file__), "resources", "test_file1.txt")
    dst_filename = join(dirname(__file__), "resources", "test_file2.txt")
    await aerofs.os.symlink(src_filename, dst_filename)
    assert await aerofs.os.path.islink(dst_filename)
    await aerofs.os.remove(dst_filename)


async def test_ismount():
    """Test the path.ismount call."""
    filename = join(dirname(__file__), "resources")
    assert not await aerofs.os.path.ismount(filename)
    assert await aerofs.os.path.ismount("/")


async def test_getsize():
    """Test path.getsize call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.getsize(filename)
    assert result == 10


async def test_samefile():
    """Test path.samefile call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.samefile(filename, filename)
    assert result


async def test_sameopenfile():
    """Test path.samefile call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.samefile(filename, filename)
    assert result


async def test_getmtime():
    """Test path.getmtime call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.getmtime(filename)
    assert result


async def test_getatime():
    """Test path.getatime call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.getatime(filename)
    assert result


async def test_getctime():
    """Test path. call."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.getctime(filename)
    assert result


async def test_link():
    """Test the link call."""
    src_filename = join(dirname(__file__), "resources", "test_file1.txt")
    dst_filename = join(dirname(__file__), "resources", "test_file2.txt")
    initial_src_nlink = stat(src_filename).st_nlink
    await aerofs.os.link(src_filename, dst_filename)
    assert (
        exists(src_filename)
        and exists(dst_filename)
        and (stat(src_filename).st_ino == stat(dst_filename).st_ino)
        and (stat(src_filename).st_nlink == initial_src_nlink + 1)
        and (stat(dst_filename).st_nlink == 2)
    )
    await aerofs.os.remove(dst_filename)
    assert (
        exists(src_filename)
        and exists(dst_filename) is False
        and (stat(src_filename).st_nlink == initial_src_nlink)
    )


async def test_symlink():
    """Test the symlink call."""
    src_filename = join(dirname(__file__), "resources", "test_file1.txt")
    dst_filename = join(dirname(__file__), "resources", "test_file2.txt")
    await aerofs.os.symlink(src_filename, dst_filename)
    assert (
        exists(src_filename)
        and exists(dst_filename)
        and stat(src_filename).st_ino == stat(dst_filename).st_ino
    )
    await aerofs.os.remove(dst_filename)
    assert exists(src_filename) and exists(dst_filename) is False


@pytest.mark.skipif(
    platform.system() == "Windows", reason="Doesn't work on Win properly"
)
async def test_readlink():
    """Test the readlink call."""
    src_filename = join(dirname(__file__), "resources", "test_file1.txt")
    dst_filename = join(dirname(__file__), "resources", "test_file2.txt")
    await aerofs.os.symlink(src_filename, dst_filename)
    symlinked_path = await aerofs.os.readlink(dst_filename)
    assert src_filename == symlinked_path
    await aerofs.os.remove(dst_filename)


async def test_listdir_empty_dir():
    """Test the listdir call when the dir is empty."""
    directory = join(dirname(__file__), "resources", "empty_dir")
    await aerofs.os.mkdir(directory)
    dir_list = await aerofs.os.listdir(directory)
    assert dir_list == []
    await aerofs.os.rmdir(directory)


async def test_listdir_dir_with_only_one_file():
    """Test the listdir call when the dir has one file."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    some_file = join(some_dir, "some_file.txt")
    await aerofs.os.mkdir(some_dir)
    with open(some_file, "w") as f:
        f.write("Test file")
    dir_list = await aerofs.os.listdir(some_dir)
    assert "some_file.txt" in dir_list
    await aerofs.os.remove(some_file)
    await aerofs.os.rmdir(some_dir)


async def test_listdir_dir_with_only_one_dir():
    """Test the listdir call when the dir has one dir."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    other_dir = join(some_dir, "other_dir")
    await aerofs.os.mkdir(some_dir)
    await aerofs.os.mkdir(other_dir)
    dir_list = await aerofs.os.listdir(some_dir)
    assert "other_dir" in dir_list
    await aerofs.os.rmdir(other_dir)
    await aerofs.os.rmdir(some_dir)


async def test_listdir_dir_with_multiple_files():
    """Test the listdir call when the dir has multiple files."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    some_file = join(some_dir, "some_file.txt")
    other_file = join(some_dir, "other_file.txt")
    await aerofs.os.mkdir(some_dir)
    with open(some_file, "w") as f:
        f.write("Test file")
    with open(other_file, "w") as f:
        f.write("Test file")
    dir_list = await aerofs.os.listdir(some_dir)
    assert "some_file.txt" in dir_list
    assert "other_file.txt" in dir_list
    await aerofs.os.remove(some_file)
    await aerofs.os.remove(other_file)
    await aerofs.os.rmdir(some_dir)


async def test_listdir_dir_with_a_file_and_a_dir():
    """Test the listdir call when the dir has files and other dirs."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    other_dir = join(some_dir, "other_dir")
    some_file = join(some_dir, "some_file.txt")
    await aerofs.os.mkdir(some_dir)
    await aerofs.os.mkdir(other_dir)
    with open(some_file, "w") as f:
        f.write("Test file")
    dir_list = await aerofs.os.listdir(some_dir)
    assert "some_file.txt" in dir_list
    assert "other_dir" in dir_list
    await aerofs.os.remove(some_file)
    await aerofs.os.rmdir(other_dir)
    await aerofs.os.rmdir(some_dir)


async def test_listdir_non_existing_dir():
    """Test the listdir call when the dir doesn't exist."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    with pytest.raises(FileNotFoundError):
        await aerofs.os.listdir(some_dir)


async def test_scantdir_empty_dir():
    """Test the scandir call when the dir is empty."""
    empty_dir = join(dirname(__file__), "resources", "empty_dir")
    await aerofs.os.mkdir(empty_dir)
    dir_iterator = await aerofs.os.scandir(empty_dir)
    dir_list = []
    for dir_entity in dir_iterator:
        dir_list.append(dir_entity)
    assert dir_list == []
    await aerofs.os.rmdir(empty_dir)


async def test_scandir_dir_with_only_one_file():
    """Test the scandir call when the dir has one file."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    some_file = join(some_dir, "some_file.txt")
    await aerofs.os.mkdir(some_dir)
    with open(some_file, "w") as f:
        f.write("Test file")
    dir_iterator = await aerofs.os.scandir(some_dir)
    some_file_entity = next(dir_iterator)
    assert some_file_entity.name == "some_file.txt"
    await aerofs.os.remove(some_file)
    await aerofs.os.rmdir(some_dir)


async def test_scandir_dir_with_only_one_dir():
    """Test the scandir call when the dir has one dir."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    other_dir = join(some_dir, "other_dir")
    await aerofs.os.mkdir(some_dir)
    await aerofs.os.mkdir(other_dir)
    dir_iterator = await aerofs.os.scandir(some_dir)
    other_dir_entity = next(dir_iterator)
    assert other_dir_entity.name == "other_dir"
    await aerofs.os.rmdir(other_dir)
    await aerofs.os.rmdir(some_dir)


async def test_scandir_non_existing_dir():
    """Test the scandir call when the dir doesn't exist."""
    some_dir = join(dirname(__file__), "resources", "some_dir")
    with pytest.raises(FileNotFoundError):
        await aerofs.os.scandir(some_dir)


@pytest.mark.skipif(platform.system() == "Windows", reason="Doesn't work on Win")
async def test_access():
    temp_file = Path(__file__).parent.joinpath("resources", "os_access_temp.txt")
    temp_dir = Path(__file__).parent.joinpath("resources", "os_access_temp")

    # prepare
    if temp_file.exists():
        os.remove(temp_file)
    assert not temp_file.exists()
    temp_file.touch()

    if temp_dir.exists():
        os.rmdir(temp_dir)
    assert not temp_dir.exists()
    os.mkdir(temp_dir)

    data = [
        # full access
        [0o777, os.F_OK, True],
        [0o777, os.R_OK, True],
        [0o777, os.W_OK, True],
        [0o777, os.X_OK, True],
        # chmod -x
        [0o666, os.F_OK, True],
        [0o666, os.R_OK, True],
        [0o666, os.W_OK, True],
        [0o666, os.X_OK, False],
        # chmod -w
        [0o444, os.F_OK, True],
        [0o444, os.R_OK, True],
        [0o444, os.W_OK, False],
        [0o444, os.X_OK, False],
        # chmod -r
        [0o000, os.F_OK, True],
        [0o000, os.R_OK, False],
        [0o000, os.W_OK, False],
        [0o000, os.X_OK, False],
    ]
    for ch, mode, access in data:
        print(f"mode:{mode}, access:{access}")
        temp_file.chmod(ch)
        temp_dir.chmod(ch)
        assert await aerofs.os.access(temp_file, mode) == access
        assert await aerofs.os.access(temp_dir, mode) == access

    # not exists
    os.remove(temp_file)
    os.rmdir(temp_dir)
    for mode in [os.F_OK, os.R_OK, os.W_OK, os.X_OK]:
        print(f"mode:{mode}")
        assert not await aerofs.os.access(temp_file, mode)
        assert not await aerofs.os.access(temp_dir, mode)


async def test_getcwd():
    """Test the getcwd call."""
    cwd = await aerofs.os.getcwd()
    assert cwd == os.getcwd()


async def test_abspath_given_abspath():
    """Test the abspath call with an absolute path."""
    filename = join(dirname(__file__), "resources", "test_file1.txt")
    file_abs_path = await aerofs.os.path.abspath(filename)
    assert file_abs_path == filename


async def test_abspath():
    """Test the abspath call."""
    relative_filename = "./tests/resources/test_file1.txt"
    abs_filename = join(dirname(__file__), "resources", "test_file1.txt")
    result = await aerofs.os.path.abspath(relative_filename)
    assert result == abs_filename



import asyncio
import aerofs


async def main():
    f = aerofs.tempfile.SpooledTemporaryFile(max_size=1024, mode="w+b")

    await f.write(b"small data")
    await f.seek(0)
    content = await f.read()
    print(f"content: {content}")

    await f.close()


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.open("trunc.txt", "w") as f:
        await f.write("this is a long string that will be truncated")

    async with aerofs.open("trunc.txt", "r+") as f:
        print(f"before: {await f.read()}")

        await f.truncate(10)
        await f.seek(0)
        print(f"after truncate(10): {await f.read()}")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    # create nested dirs
    await aerofs.os.makedirs("test_dir/sub/deep", exist_ok=True)
    print("created nested directories")

    # list directory
    entries = await aerofs.os.listdir("test_dir")
    print(f"listdir: {entries}")

    # cleanup
    await aerofs.os.rmdir("test_dir/sub/deep")
    await aerofs.os.rmdir("test_dir/sub")
    await aerofs.os.rmdir("test_dir")
    print("cleaned up")


if __name__ == "__main__":
    asyncio.run(main())



"""Type stubs for aerofs.os module."""

from os import stat_result
from typing import Iterator, Literal, Optional, Union
from os import PathLike

class DirEntry:
    """Directory entry from scandir."""
    
    @property
    def name(self) -> str: ...
    @property
    def path(self) -> str: ...
    def is_dir(self) -> bool: ...
    def is_file(self) -> bool: ...

async def stat(path: Union[str, PathLike[str]]) -> stat_result: ...
async def remove(path: str) -> None: ...
async def unlink(path: str) -> None: ...
async def rename(src: str, dst: str) -> None: ...
async def replace(src: str, dst: str) -> None: ...
async def renames(old: str, new: str) -> None: ...
async def removedirs(name: str) -> None: ...

async def mkdir(path: str, mode: Optional[int] = None) -> None: ...
async def makedirs(
    path: str, mode: Optional[int] = None, exist_ok: bool = False
) -> None: ...
async def rmdir(path: str) -> None: ...

async def listdir(path: Optional[str] = None) -> list[str]: ...
async def scandir(path: Optional[str] = None) -> Iterator[DirEntry]: ...

async def getcwd() -> str: ...

async def access(
    path: Union[str, PathLike[str]],
    mode: Literal[0, 1, 2, 4],
) -> bool: ...

# Unix-only functions
async def link(src: str, dst: str) -> None: ...
async def symlink(src: str, dst: str) -> None: ...
async def readlink(path: str) -> str: ...
async def statvfs(path: str) -> object: ...
async def sendfile(
    out_fd: int, in_fd: int, offset: Optional[int], count: int
) -> int: ...

class _PathModule:
    """Async path operations module."""
    
    async def exists(self, path: str) -> bool: ...
    async def isfile(self, path: str) -> bool: ...
    async def isdir(self, path: str) -> bool: ...
    async def islink(self, path: str) -> bool: ...
    async def getsize(self, path: str) -> int: ...
    async def abspath(self, path: str) -> str: ...
    async def getmtime(self, path: str) -> float: ...
    async def getatime(self, path: str) -> float: ...
    async def getctime(self, path: str) -> float: ...
    async def ismount(self, path: str) -> bool: ...
    async def samefile(self, path1: str, path2: str) -> bool: ...
    async def sameopenfile(self, fd1: int, fd2: int) -> bool: ...

# Note: These are actually module-level functions, exposed as path.exists() etc.
async def exists(path: str) -> bool: ...
async def isfile(path: str) -> bool: ...
async def isdir(path: str) -> bool: ...
async def islink(path: str) -> bool: ...
async def getsize(path: str) -> int: ...
async def abspath(path: str) -> str: ...
async def getmtime(path: str) -> float: ...
async def getatime(path: str) -> float: ...
async def getctime(path: str) -> float: ...
async def ismount(path: str) -> bool: ...
async def samefile(path1: str, path2: str) -> bool: ...
async def sameopenfile(fd1: int, fd2: int) -> bool: ...

class path:
    """Submodule containing async path operations."""
    
    @staticmethod
    async def exists(path: str) -> bool: ...
    @staticmethod
    async def isfile(path: str) -> bool: ...
    @staticmethod
    async def isdir(path: str) -> bool: ...
    @staticmethod
    async def islink(path: str) -> bool: ...
    @staticmethod
    async def getsize(path: str) -> int: ...
    @staticmethod
    async def abspath(path: str) -> str: ...
    @staticmethod
    async def getmtime(path: str) -> float: ...
    @staticmethod
    async def getatime(path: str) -> float: ...
    @staticmethod
    async def getctime(path: str) -> float: ...
    @staticmethod
    async def ismount(path: str) -> bool: ...
    @staticmethod
    async def samefile(path1: str, path2: str) -> bool: ...
    @staticmethod
    async def sameopenfile(fd1: int, fd2: int) -> bool: ...

__all__: list[str]



"""
aerofs: High-Performance Asynchronous File I/O for Python, powered by Rust.
"""

from typing import (
    Any,
    AnyStr,
    AsyncIterator,
    Literal,
    Optional,
    Union,
    overload,
)
from os import PathLike

__version__: str
__author__: str
__all__: list[str]

class AsyncFile:
    """Asynchronous file object returned by aerofs.open()."""
    
    closed: bool
    name: str
    mode: str
    
    async def __aenter__(self) -> "AsyncFile": ...
    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[Any],
    ) -> None: ...
    def __aiter__(self) -> AsyncIterator[Union[str, bytes]]: ...
    async def __anext__(self) -> Union[str, bytes]: ...
    
    async def read(self, size: Optional[int] = None) -> Union[str, bytes]: ...
    async def read1(self, size: Optional[int] = None) -> Union[str, bytes]: ...
    async def readall(self) -> Union[str, bytes]: ...
    async def readline(self, size: Optional[int] = None) -> Union[str, bytes]: ...
    async def readlines(self, hint: Optional[int] = None) -> list[Union[str, bytes]]: ...
    async def readinto(self, buffer: bytearray) -> int: ...
    
    async def write(self, data: Union[str, bytes]) -> int: ...
    async def writelines(self, lines: list[Union[str, bytes]]) -> None: ...
    
    async def seek(self, offset: int, whence: int = 0) -> int: ...
    async def tell(self) -> int: ...
    async def truncate(self, size: Optional[int] = None) -> int: ...
    async def flush(self) -> None: ...
    async def close(self) -> None: ...
    
    async def peek(self, size: Optional[int] = None) -> Union[str, bytes]: ...
    def fileno(self) -> int: ...
    def detach(self) -> Any: ...
    
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def isatty(self) -> bool: ...

class AsyncStdin:
    """Asynchronous stdin reader."""
    async def read(self) -> None:
        """Not supported in async context. Raises OSError."""
        ...
    async def readline(self) -> Union[str, bytes]: ...

class AsyncStdout:
    """Asynchronous stdout writer."""
    async def write(self, data: Union[str, bytes]) -> int: ...
    async def flush(self) -> None: ...

class AsyncStderr:
    """Asynchronous stderr writer."""
    async def write(self, data: Union[str, bytes]) -> int: ...
    async def flush(self) -> None: ...

stdin: AsyncStdin
stdout: AsyncStdout
stderr: AsyncStderr
stdin_bytes: AsyncStdin
stdout_bytes: AsyncStdout
stderr_bytes: AsyncStderr

@overload
def open(
    file: Union[str, PathLike[str]],
    mode: Literal["r", "r+", "w", "w+", "a", "a+", "x", "x+"] = "r",
    buffering: int = -1,
    encoding: Optional[str] = None,
    errors: Optional[str] = None,
    newline: Optional[str] = None,
    closefd: bool = True,
    opener: Optional[Any] = None,
    loop: Optional[Any] = None,
    executor: Optional[Any] = None,
) -> AsyncFile: ...

@overload
def open(
    file: Union[str, PathLike[str]],
    mode: Literal["rb", "rb+", "r+b", "wb", "wb+", "w+b", "ab", "ab+", "a+b", "xb", "xb+", "x+b"],
    buffering: int = -1,
    encoding: Optional[str] = None,
    errors: Optional[str] = None,
    newline: Optional[str] = None,
    closefd: bool = True,
    opener: Optional[Any] = None,
    loop: Optional[Any] = None,
    executor: Optional[Any] = None,
) -> AsyncFile: ...

def open(
    file: Union[str, PathLike[str]],
    mode: str = "r",
    buffering: int = -1,
    encoding: Optional[str] = None,
    errors: Optional[str] = None,
    newline: Optional[str] = None,
    closefd: bool = True,
    opener: Optional[Any] = None,
    loop: Optional[Any] = None,
    executor: Optional[Any] = None,
) -> AsyncFile:
    """
    Open a file asynchronously.
    
    This is a drop-in replacement for the built-in open() function,
    returning an AsyncFile object that supports async context managers
    and async iteration.
    """
    ...

from . import os as os
from . import tempfile as tempfile
from . import threadpool as threadpool



import asyncio
from pathlib import Path

import pytest

from aerofs.threadpool import open as aioopen

RESOURCES_DIR = Path(__file__).parent.parent / "resources"
TEST_FILE = RESOURCES_DIR / "test_file1.txt"
TEST_FILE_CONTENTS = "0123456789"


@pytest.mark.parametrize("mode", ["r", "rb"])
async def test_file_not_found(mode):
    filename = "non_existent"

    try:
        open(filename, mode=mode)
    except Exception as e:
        expected = e

    assert expected

    try:
        await aioopen(filename, mode=mode)
    except Exception as e:
        actual = e

    assert actual

    assert actual.errno == expected.errno
    assert str(actual) == str(expected)


async def test_file_async_context_aexit():
    async with aioopen(TEST_FILE) as fp:
        pass

    with pytest.raises(ValueError):
        line = await fp.read()

    async with aioopen(TEST_FILE) as fp:
        line = await fp.read()
        assert line == TEST_FILE_CONTENTS


async def test_filetask_async_context_aexit():
    async def _process_test_file(file_ctx, sleep_time: float = 1.0):
        nonlocal file_ref
        async with file_ctx as fp:
            file_ref = file_ctx._obj
            await asyncio.sleep(sleep_time)
            await fp.read()

    cancel_time, sleep_time = 0.1, 10
    assert cancel_time <= (sleep_time / 10)

    file_ref = None
    file_ctx = aioopen(TEST_FILE)

    task = asyncio.create_task(
        _process_test_file(file_ctx=file_ctx, sleep_time=sleep_time)
    )
    try:
        await asyncio.wait_for(task, timeout=cancel_time)
    except asyncio.TimeoutError:
        assert task.cancelled

    assert file_ref.closed



import asyncio
import aerofs


async def main():
    async with aerofs.open("props.txt", "w") as f:
        print(f"name: {f.name}")
        print(f"mode: {f.mode}")
        print(f"closed: {f.closed}")
        print(f"readable: {f.readable()}")
        print(f"writable: {f.writable()}")
        print(f"seekable: {f.seekable()}")

    await aerofs.os.remove("props.txt")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    await aerofs.os.makedirs("scan_test", exist_ok=True)

    async with aerofs.open("scan_test/file1.txt", "w") as f:
        await f.write("hello")
    async with aerofs.open("scan_test/file2.txt", "w") as f:
        await f.write("world")

    entries = await aerofs.os.scandir("scan_test")
    for entry in entries:
        print(f"{entry.name}: is_file={entry.is_file()}, is_dir={entry.is_dir()}")

    # cleanup
    await aerofs.os.remove("scan_test/file1.txt")
    await aerofs.os.remove("scan_test/file2.txt")
    await aerofs.os.rmdir("scan_test")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import os
import aerofs


async def main():
    async with aerofs.open("access_test.txt", "w") as f:
        await f.write("test")

    print(f"F_OK (exists): {await aerofs.os.access('access_test.txt', os.F_OK)}")
    print(f"R_OK (readable): {await aerofs.os.access('access_test.txt', os.R_OK)}")
    print(f"W_OK (writable): {await aerofs.os.access('access_test.txt', os.W_OK)}")
    print(f"X_OK (executable): {await aerofs.os.access('access_test.txt', os.X_OK)}")

    await aerofs.os.remove("access_test.txt")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import aerofs


async def main():
    async with aerofs.open("original.txt", "w") as f:
        await f.write("testing rename")

    await aerofs.os.rename("original.txt", "renamed.txt")
    print("renamed to renamed.txt")

    exists = await aerofs.os.path.exists("renamed.txt")
    print(f"renamed.txt exists: {exists}")

    await aerofs.os.remove("renamed.txt")
    print("removed")


if __name__ == "__main__":
    asyncio.run(main())
